@@ -28,6 +28,40 @@ from synqed.display import MessageDisplay
 logger = logging.getLogger(__name__)
 
 
+def infer_turn_type(message_content: str) -> str:
+    """
+    Infer the turn type from a message based on heuristics.
+    
+    This helps classify agent actions into categories:
+    - delegation: Agent is delegating work to another agent
+    - finalization: Agent is providing a final result
+    - proposal: Agent is proposing a solution or idea
+    - coordination: Agent is coordinating with others
+    
+    Args:
+        message_content: The content of the message
+        
+    Returns:
+        Turn type string (delegation, finalization, proposal, coordination)
+    """
+    content_lower = message_content.lower()
+    
+    # Check for delegation keywords
+    if any(word in content_lower for word in ["delegate", "please handle", "can you", "could you", "need your help"]):
+        return "delegation"
+    
+    # Check for finalization keywords
+    if any(word in content_lower for word in ["final", "complete", "done", "finished", "ready for user", "here is the result"]):
+        return "finalization"
+    
+    # Check for coordination keywords
+    if any(word in content_lower for word in ["coordinate", "collaborate", "work together", "let's", "we should"]):
+        return "coordination"
+    
+    # Default to proposal
+    return "proposal"
+
+
 class Context(AgentLogicContext):
     """
     Context object passed to agent logic functions during workspace execution.
@@ -45,13 +79,15 @@ class Context(AgentLogicContext):
         default_target: Optional[str] = None,
         event_trigger: Optional[str] = None,
         event_payload: Optional[dict] = None,
+        shared_plan: Optional[str] = None,
     ):
         """Initialize the context."""
         super().__init__(
             memory=memory, 
             default_target=default_target,
             workspace=workspace,
-            agent_name=agent_name
+            agent_name=agent_name,
+            shared_plan=shared_plan
         )
         self.agent_name = agent_name
         self.workspace = workspace
@@ -132,6 +168,8 @@ class WorkspaceExecutionEngine:
         self.global_workspace_queue: asyncio.Queue[str] = asyncio.Queue()
         # Track which workspaces are currently executing to prevent re-entry
         self._running_workspaces: set[str] = set()
+        # Track which workspaces are queued to prevent duplicate scheduling
+        self._queued_workspaces: set[str] = set()
         # Track which agent requested each child workspace (child_id -> requesting_agent_name)
         self._subteam_requesters: dict[str, str] = {}
         # Real-time message display
@@ -144,7 +182,7 @@ class WorkspaceExecutionEngine:
         Schedule a workspace for execution in the global queue.
         
         This method enqueues a workspace ID for later execution. Workspaces
-        cannot be enqueued if they are already running.
+        cannot be enqueued if they are already running OR already in the queue.
         
         Args:
             workspace_id: ID of workspace to schedule
@@ -156,7 +194,14 @@ class WorkspaceExecutionEngine:
             logger.debug(f"Workspace {workspace_id} is already running, skipping enqueue")
             return
         
+        # FIX: Check if workspace is already in the queue to prevent duplicate scheduling
+        # This prevents the same workspace from being queued multiple times before execution
+        if workspace_id in self._queued_workspaces:
+            logger.debug(f"Workspace {workspace_id} is already queued, skipping duplicate enqueue")
+            return
+        
         # Enqueue for execution
+        self._queued_workspaces.add(workspace_id)
         self.global_workspace_queue.put_nowait(workspace_id)
         logger.debug(f"Scheduled workspace {workspace_id} for execution")
     
@@ -191,6 +236,11 @@ class WorkspaceExecutionEngine:
             if workspace_ids:
                 logger.debug(f"Running {len(workspace_ids)} workspaces in parallel: {workspace_ids}")
                 
+                # FIX: Remove workspaces from queued set before running
+                # This allows them to be re-queued later if needed
+                for wid in workspace_ids:
+                    self._queued_workspaces.discard(wid)
+                
                 # Run all workspaces in this batch CONCURRENTLY
                 tasks = [self.run_workspace(workspace_id=wid) for wid in workspace_ids]
                 try:
@@ -260,22 +310,36 @@ class WorkspaceExecutionEngine:
             scheduler = self._workspace_schedulers[workspace_id]
         
             # Schedule startup events for all agents (only on first execution)
+            # FIX: Skip agents that already have unprocessed messages (e.g., initial USER message)
+            # FIX: Only send startup to root workspace (child workspaces activate on-demand via messages)
+            # The unprocessed messages section below will schedule events for them
             if not workspace.has_started:
-                for agent_name, agent in workspace.agents.items():
-                    # Route a system startup message via workspace
-                    msg_id = await workspace.route_message(
-                        sender="SYSTEM",
-                        recipient=agent_name,
-                        content="[startup]",
-                        manager=self.workspace_manager,
-                    )
-                    
-                    # Schedule startup event carrying that message_id
-                    scheduler.schedule_event_dedup(AgentEvent(
-                        agent_name=agent_name,
-                        trigger="startup",
-                        payload={"message_id": msg_id},
-                    ))
+                # Only root workspace agents get startup events
+                # Child workspace agents activate only when messaged
+                if workspace.depth == 0:
+                    for agent_name, agent in workspace.agents.items():
+                        # Check if agent already has unprocessed messages
+                        unprocessed = agent.memory.get_unprocessed_messages()
+                        if unprocessed:
+                            # Agent already has work to do, skip startup event
+                            # The unprocessed messages loop below will schedule events for these
+                            logger.debug(f"Skipping startup event for {agent_name} - already has {len(unprocessed)} unprocessed messages")
+                            continue
+                        
+                        # Route a system startup message via workspace
+                        msg_id = await workspace.route_message(
+                            sender="SYSTEM",
+                            recipient=agent_name,
+                            content="[startup]",
+                            manager=self.workspace_manager,
+                        )
+                        
+                        # Schedule startup event carrying that message_id
+                        scheduler.schedule_event_dedup(AgentEvent(
+                            agent_name=agent_name,
+                            trigger="startup",
+                            payload={"message_id": msg_id},
+                        ))
                 workspace.has_started = True
             
             # Check for unprocessed messages (e.g., subteam_result from child workspaces)
@@ -283,6 +347,22 @@ class WorkspaceExecutionEngine:
             for agent_name, agent in workspace.agents.items():
                 unprocessed_messages = agent.memory.get_unprocessed_messages()
                 for message in unprocessed_messages:
+                    # ROUTING ENFORCEMENT: Only schedule event if message is targeted to this agent
+                    # target should always be set by router, but check just in case
+                    message_target = getattr(message, 'target', None)
+                    
+                    # If target is set and doesn't match this agent, skip
+                    if message_target is not None and message_target != agent_name:
+                        logger.debug(f"Skipping message {message.message_id} for {agent_name} - targeted to {message_target}")
+                        continue
+                    
+                    # If target is not set, this is a legacy message or system message
+                    # Only allow for the first agent (backwards compatibility) or if content suggests it's for this agent
+                    if message_target is None:
+                        # For None targets, only schedule if this is a system message or workspace-level broadcast
+                        # Most messages should have targets, so log a warning
+                        logger.warning(f"Message {message.message_id} has no target, scheduling for {agent_name}")
+                    
                     # Determine trigger type based on message content
                     trigger = "message"
                     if message.content.startswith("[subteam_result]"):
@@ -333,10 +413,8 @@ class WorkspaceExecutionEngine:
                     total_events_processed += 1
                     agent_name = event.agent_name
                     
-                    # Display processing message when agent changes
-                    if self.display and agent_name != self._active_agent:
-                        self._active_agent = agent_name
-                        self.display.display_processing(agent_name)
+                    # NOTE: Display processing message AFTER we confirm agent produces output
+                    # (moved below to avoid showing "processing" for filtered messages)
                     
                     # Strict validation for event.trigger
                     valid_triggers = {"startup", "message", "subteam_result"}
@@ -385,7 +463,7 @@ class WorkspaceExecutionEngine:
                     # Get messages for context
                     messages = agent.memory.get_messages()
                     
-                    # Create context with event trigger and payload
+                    # Create context with event trigger, payload, and shared_plan
                     context = Context(
                         agent_name=agent_name,
                         workspace=workspace,
@@ -395,6 +473,7 @@ class WorkspaceExecutionEngine:
                         default_target=agent.default_target,
                         event_trigger=event.trigger,
                         event_payload=event.payload,
+                        shared_plan=workspace.shared_plan,
                     )
                     
                     try:
@@ -410,10 +489,25 @@ class WorkspaceExecutionEngine:
                             # Mark message as processed after successful logic execution
                             if message_id:
                                 agent.memory.mark_message_processed(message_id)
+                            
+                            # Check if agent returned None (agent chose not to respond)
+                            if result is None:
+                                logger.debug(f"Agent {agent_name} returned None - skipping (no response needed)")
+                                continue
                         else:
                             # Remote A2A agent - call get_response() (messages already buffered)
                             result = await agent.get_response()
                             # No memory to mark - remote agent manages its own state
+                            
+                            # Check if agent returned None (e.g., for startup messages that should be skipped)
+                            if result is None:
+                                logger.debug(f"Agent {agent_name} returned None - skipping (no response needed)")
+                                continue
+                        
+                        # Display processing message NOW (agent produced output)
+                        if self.display and agent_name != self._active_agent:
+                            self._active_agent = agent_name
+                            self.display.display_processing(agent_name)
                         
                         # Hardened agent output validation (supports broadcast)
                         agent_response = self._validate_and_normalize_response(
@@ -428,6 +522,13 @@ class WorkspaceExecutionEngine:
                         
                         # Process each response (single or broadcast)
                         for single_response in responses_to_process:
+                            # Infer turn type for this response
+                            turn_type = infer_turn_type(single_response.get("content", ""))
+                            logger.debug(f"Agent {agent_name} turn type: {turn_type}")
+                            
+                            # Optionally update shared plan based on turn type
+                            # (For now, we just log it; agents can update shared_plan organically)
+                            
                             # Handle USER messages: route and STOP execution (task complete)
                             if single_response["send_to"] == "USER":
                                 # Display the message
