@@ -459,6 +459,10 @@ class PlannerLLM:
         """
         Break a user task into a hierarchical task tree with agent assignments.
         
+        This method automatically queries the AgentRuntimeRegistry to discover
+        available agents and their capabilities, ensuring only registered agents
+        are assigned to tasks.
+        
         Args:
             user_task: The user's task description
             task_id: Optional unique identifier for the task
@@ -472,39 +476,80 @@ class PlannerLLM:
         import uuid
         task_id = task_id or str(uuid.uuid4())
         
-        system_prompt = """You are a hierarchical task planning expert. Break the user's task into a recursive tree structure.
+        # Query AgentRuntimeRegistry to get available agents
+        from synqed.workspace_manager import AgentRuntimeRegistry
+        
+        available_agents = []
+        for role in AgentRuntimeRegistry.list_roles():
+            agent = AgentRuntimeRegistry.get(role)
+            if agent:
+                # Get agent capabilities from agent metadata
+                caps = agent.capabilities if agent.capabilities else ["general tasks"]
+                available_agents.append({
+                    "name": role,
+                    "description": agent.description,
+                    "capabilities": caps
+                })
+        
+        # Build agent roster for system prompt
+        if available_agents:
+            agent_roster = "\n".join([
+                f"  - {a['name']}: {a['description']}\n    Capabilities: {', '.join(a['capabilities'])}"
+                for a in available_agents
+            ])
+            agent_constraint = f"""
+AVAILABLE AGENTS (you MUST ONLY use these agents):
+{agent_roster}
+
+CRITICAL: Only assign tasks to the agents listed above. Match agent capabilities to task requirements."""
+        else:
+            agent_constraint = "Note: No agents registered yet. Use generic role names."
+        
+        system_prompt = f"""You are a hierarchical task planning expert. Break the user's task into a recursive tree structure.
+
+{agent_constraint}
 
 Respond with a SINGLE JSON object representing the ROOT TaskTreeNode ONLY:
 
-{
+{{
   "id": "placeholder",
   "description": "overall task description",
-  "required_agents": ["Coordinator"],
+  "required_agents": ["agent_name1", "agent_name2"],
   "may_need_subteams": true,
   "children": [
-    {
+    {{
       "id": "placeholder",
       "description": "subtask A description",
-      "required_agents": ["Writer", "Editor"],
+      "required_agents": ["agent_name1", "agent_name2"],
       "may_need_subteams": true,
       "children": [
-        {
+        {{
           "id": "placeholder",
           "description": "subtask A1 description",
-          "required_agents": ["Researcher"],
+          "required_agents": ["agent_name3", "agent_name4"],
           "may_need_subteams": false,
           "children": []
-        }
+        }}
       ]
-    }
+    }},
+    {{
+      "id": "placeholder",
+      "description": "subtask B description",
+      "required_agents": ["agent_name5", "agent_name6"],
+      "may_need_subteams": false,
+      "children": []
+    }}
   ]
-}
+}}
 
 RULES:
 - Output ONLY a single JSON object representing the root node.
 - Do NOT include task_id, original_task, wrapper objects, or extra fields.
 - Every node MUST have: id, description, required_agents, may_need_subteams (optional), children.
 - Children MUST follow the exact same schema recursively.
+- ONLY use agent names from the AVAILABLE AGENTS list above.
+- Match agent capabilities to task requirements when assigning agents.
+- MINIMUM 2 AGENTS per subtask/workspace: Each child node should have at least 2 agents assigned to enable collaboration. Pair agents with complementary capabilities.
 - Ignore IDs; placeholder IDs are acceptable, they will be replaced by the system. Focus on correct tree structure.
 - Output NOTHING except valid JSON."""
         
@@ -513,6 +558,9 @@ RULES:
 Task: {user_task}
 
 Create a recursive tree structure where complex tasks are broken into subtasks, and those subtasks can have their own children.
+Assign agents from the available list based on their capabilities.
+
+IMPORTANT: Each subtask should have MINIMUM 2 AGENTS to enable collaboration and coordination. Pair agents with complementary skills (e.g., a coordinator with a specialist, a manager with an executor).
 
 Respond with valid JSON only."""
         
@@ -865,6 +913,164 @@ Respond with valid JSON only."""
         else:
             raise ValueError(f"Unsupported provider: {self.provider}")
     
+    async def create_agents_from_task(
+        self,
+        user_task: str,
+        provider: str = "anthropic",
+        api_key: Optional[str] = None,
+        model: Optional[str] = None,
+    ) -> list[dict[str, Any]]:
+        """
+        Dynamically create agent specifications based on the user task.
+        
+        This method analyzes the task and determines what agents are needed,
+        then generates agent specifications including name, description, 
+        capabilities, and role.
+        
+        Args:
+            user_task: The user's task description
+            provider: LLM provider for generated agents ("anthropic" or "openai")
+            api_key: API key for the agent LLM (defaults to planner's API key)
+            model: Model name for generated agents (defaults to planner's model)
+            
+        Returns:
+            List of agent specifications, each containing:
+            {
+                "name": str,
+                "description": str,
+                "capabilities": list[str],
+                "role": str,
+                "provider": str,
+                "api_key": str,
+                "model": str
+            }
+            
+        Example:
+            ```python
+            planner = PlannerLLM(provider="anthropic", api_key="...")
+            
+            agent_specs = await planner.create_agents_from_task(
+                "Plan a wedding with vendors, budget, and timeline"
+            )
+            
+            # agent_specs will contain specifications for agents like:
+            # - wedding_planner: coordinates overall planning
+            # - vendor_coordinator: manages vendor relationships
+            # - budget_manager: tracks expenses and budget
+            # - timeline_coordinator: manages schedules and deadlines
+            ```
+        """
+        import uuid
+        
+        # Use provided credentials or fall back to planner's credentials
+        agent_provider = provider
+        agent_api_key = api_key or self.api_key
+        agent_model = model or self.model
+        
+        system_prompt = """You are an expert at designing multi-agent systems. Given a task, determine what agents are needed to accomplish it.
+
+For each agent, provide:
+- name: snake_case identifier (e.g., "research_specialist")
+- description: Clear description of the agent's purpose
+- capabilities: List of specific capabilities/skills
+- role: A team/domain identifier (e.g., "research_team", "engineering_team")
+
+Output a JSON array of agent specifications:
+
+[
+  {
+    "name": "agent_name",
+    "description": "Agent description",
+    "capabilities": ["capability1", "capability2", "capability3"],
+    "role": "team_name"
+  }
+]
+
+RULES:
+- Create 2-6 agents depending on task complexity
+- Each agent should have a clear, distinct purpose
+- Agents should be able to collaborate and delegate work
+- Include a coordinator/manager agent to orchestrate the work
+- Use descriptive, professional names
+- Capabilities should be specific and actionable
+- Output ONLY valid JSON, nothing else"""
+        
+        user_prompt = f"""Analyze this task and design a multi-agent system to accomplish it:
+
+Task: {user_task}
+
+Create agent specifications that will work together effectively. Include agents with complementary skills.
+Respond with valid JSON only."""
+        
+        # Make LLM call
+        response_text = await self._call_llm(system_prompt, user_prompt)
+        
+        # Parse and validate response
+        try:
+            # Extract JSON from response
+            json_str = self.extract_json(response_text)
+            
+            # Handle both array and object responses
+            parsed = json.loads(json_str)
+            
+            # If response is wrapped in an object, extract the array
+            if isinstance(parsed, dict):
+                # Try common wrapper keys
+                for key in ["agents", "agent_specs", "specifications", "agent_list"]:
+                    if key in parsed and isinstance(parsed[key], list):
+                        parsed = parsed[key]
+                        break
+                else:
+                    # If no wrapper found but it's a single agent spec, wrap it
+                    if "name" in parsed and "description" in parsed:
+                        parsed = [parsed]
+                    else:
+                        raise ValueError("Response is a dict but doesn't contain agent specifications")
+            
+            if not isinstance(parsed, list):
+                raise ValueError(f"Expected list of agents, got {type(parsed)}")
+            
+            # Validate and enrich agent specifications
+            agent_specs = []
+            for i, agent_spec in enumerate(parsed):
+                if not isinstance(agent_spec, dict):
+                    logger.warning(f"Skipping invalid agent spec at index {i}: not a dict")
+                    continue
+                
+                # Validate required fields
+                if "name" not in agent_spec or "description" not in agent_spec:
+                    logger.warning(f"Skipping invalid agent spec at index {i}: missing name or description")
+                    continue
+                
+                # Ensure capabilities is a list
+                if "capabilities" not in agent_spec:
+                    agent_spec["capabilities"] = ["general tasks"]
+                elif not isinstance(agent_spec["capabilities"], list):
+                    agent_spec["capabilities"] = [str(agent_spec["capabilities"])]
+                
+                # Ensure role is set
+                if "role" not in agent_spec or not agent_spec["role"]:
+                    agent_spec["role"] = "team"
+                
+                # Add LLM credentials
+                agent_spec["provider"] = agent_provider
+                agent_spec["api_key"] = agent_api_key
+                agent_spec["model"] = agent_model
+                
+                agent_specs.append(agent_spec)
+            
+            if not agent_specs:
+                raise ValueError("No valid agent specifications generated")
+            
+            logger.info(f"Generated {len(agent_specs)} agent specifications from task")
+            return agent_specs
+            
+        except (json.JSONDecodeError, ValueError) as e:
+            logger.error(f"Failed to parse agent specifications: {e}")
+            raise ValueError(
+                f"Invalid agent specification response. Received: {response_text[:200]}"
+            ) from e
+    
     def __repr__(self) -> str:
         """String representation."""
         return f"PlannerLLM(provider='{self.provider}', model='{self.model}')"
