@@ -114,11 +114,11 @@ class MessageRouter:
         # Check if this is a local agent or remote A2A agent
         if hasattr(agent, 'memory'):
             # Local agent - direct memory access
-            msg_id = agent.memory.add_message(from_agent=sender, content=content, message_id=message_id)
+            msg_id = agent.memory.add_message(from_agent=sender, content=content, message_id=message_id, target=recipient)
         else:
             # Remote A2A agent - buffer message via send_message()
             from synqed.memory import InboxMessage
-            inbox_msg = InboxMessage(from_agent=sender, content=content)
+            inbox_msg = InboxMessage(from_agent=sender, content=content, target=recipient)
             await agent.send_message(inbox_msg)
             # Generate message_id for transcript
             msg_id = message_id or f"msg-{workspace_id}-{recipient}-{id(inbox_msg)}"
