@@ -171,9 +171,12 @@ class AgentRuntimeRegistry:
             from synqed.a2a_remote_agent import RemoteA2AAgent
             
             # Create new RemoteA2AAgent instance
+            # IMPORTANT: Use role as the name for routing consistency
+            # Workspace.add_agent() keys agents by agent.name, so we must use the role name
+            # to ensure routing works correctly (agents send messages using role names)
             remote_agent = RemoteA2AAgent(
                 url=remote_config["url"],
-                name=remote_config.get("name", role),
+                name=role,  # Use role as name for routing consistency
                 auth_token=remote_config.get("auth_token"),
                 transport=remote_config.get("transport", "JSONRPC"),
                 description=remote_config.get("description")
@@ -250,6 +253,7 @@ class Workspace(BaseModel):
     )
     is_running: bool = Field(default=False, exclude=True, description="Flag indicating if workspace is currently executing")
     has_started: bool = Field(default=False, exclude=True, description="Flag indicating if workspace has received startup events")
+    shared_plan: str = Field(default="", exclude=True, description="Shared plan document that all agents can read and update")
     
     model_config = {"arbitrary_types_allowed": True}
     
@@ -451,6 +455,21 @@ class Workspace(BaseModel):
         """
         self.message_callbacks.append(callback)
     
+    def append_to_shared_plan(self, agent_name: str, content: str) -> None:
+        """
+        Append content to the shared plan document.
+        
+        This allows agents to organically update the shared plan as they work.
+        
+        Args:
+            agent_name: Name of the agent updating the plan
+            content: Content to append to the shared plan
+        """
+        if self.shared_plan:
+            self.shared_plan += f"\n\n[{agent_name}] {content}"
+        else:
+            self.shared_plan = f"[{agent_name}] {content}"
+    
     def log_routed_message(
         self,
         sender: str,
